@@ -1,6 +1,7 @@
 import numpy as np
 import pandas as pd
 from op_nlopt.ord12 import *
+from op_nlopt.cstm import *
 from gnrl.sf import *
 from gnrl.measr import *
 
@@ -609,7 +610,7 @@ def visualz(model, iterate, dsp):
         print(pd.DataFrame({'No': No,
                             'Variable': Variable,
                             'DF': DF,
-                            'ScoreChiS': ScoreChiSq,
+                            'ScoreChiSq': ScoreChiSq,
                             'p_valScoreChiSq': p_valScoreChiSq
                             }))
         print('------------------------------------------------------')
