@@ -212,23 +212,22 @@ def dv2dm(x, cols):
             current_pos += 1
     return X
 ##############################################################################
-def elspm(U, Y, na=0, nb=0, nc=0, pm0=0, maxiter=50, axcnv=None, xcnv=1e-8, afcnv=None, fcnv=1e-8, gcnv=None,
-          dsp=False):
+def elspm(U, Y, na=0, nb=0, nc=0, pm0=0, maxiter=50, axcnv=None, xcnv=1e-8, afcnv=None, fcnv=1e-8, gcnv=None, dsp=False):
     N, r = Y.shape
     n = max(na, nb, nc)
     nn = n - max(na, nb)
 
-    Pmab = lspm(U, Y, na, nb, pm0)
-    Fab = dmpm(U[nn:], Y[nn:], na=na, nb=nb, nc=0, pm0=pm0)
-    E = np.vstack((np.zeros([n, r]), Y[n:, :] - Fab@Pmab))
+    Parx = lspm(U, Y, na, nb, pm0)
+    Farx = dmpm(U[nn:], Y[nn:], na=na, nb=nb, nc=0, pm0=pm0)
+    E = np.vstack((np.zeros([n, r]), Y[n:, :] - lin_apl(Farx, Parx)))
     f = np.sum(np.diag(E.T@E) / N)
-    if nc == 0: return Pm, E
-    Pm = np.vstack((Pmab, np.zeros([r * nc, r])))
+    # if nc == 0: return Pm, E
+    Pm = np.vstack((Parx, np.zeros([r*nc, r])))
     itr = 0
     iterate = 1
     while iterate:
         itr += 1
-        F = np.hstack((Fab, dmpm(E=E, nc=nc)))
+        F = np.hstack((Farx, dmpm(E=E, nc=nc)))
         Pm_1 = Pm
         Pm = nsinv(F.T@F)@F.T@Y[n:]
         E = np.vstack((np.zeros((n, r)), Y[n:] - F@Pm))
@@ -247,40 +246,54 @@ def elspm(U, Y, na=0, nb=0, nc=0, pm0=0, maxiter=50, axcnv=None, xcnv=1e-8, afcn
         print(maxiter, "is the maximum number of iterations.")
     return Pm, E
 ##############################################################################
-def elspm_apl(U, Y, E, Pm, na=0, nb=0, nc=0, pm0=0):
-    F = dmpm(U, Y, E, na, nb, nc, pm0)
-    Ym = np.dot(F, Pm)
-    return Ym
+def armax_apl(U, Y=None, E=None, pm=None, na=0, nb=0, nc=0, pm0=0, sim=False, ltv=False):
+    if isinstance(nb, (int, float)):
+        r = pm.shape[1]
+        m = U.shape[1]
+        mtype = 'PM'
+    else:
+        r = nb.shape[0]
+        m = nb.shape[1]
+        mtype = 'PV'
+    if mtype == 'PV': pm = pv2m(pm, na, nb, nc, pm0=pm0) # if model in PV form --> goto PM form
+    nn = np.max(np.hstack((na, nb, nc)))
+    if not sim:
+        if nn == 0:
+            if pm0: U = np.hstack((np.ones((U.shape[0], 1)), U))
+            ym = lin_apl(U, pm)
+        else:
+            F = dmpm(U, Y, E, na, nb, nc, pm0)
+            ym = lin_apl(F, pm)
+    else:
+        N = U.shape[0]
+        r = pm.shape[1]
+        if E is None: E = zeros((N, r))
+        ym = np.zeros((N, r))
+        for k in range(nn, N):
+            fi = dmpm(U[k - nn:k + 1, :], ym[k - nn:k + 1, :], E[k - nn:k + 1, :], na, nb, nc, pm0)
+            ym[k, :] = fi@pm + r_(E[k, :])
+    return ym
+
+
 ##############################################################################
-def elspv(U, Y, na=0, nb=0, nc=0, pm0=0, maxiter=50, axcnv=None, xcnv=1e-8, afcnv=None, fcnv=1e-8, gcnv=None,
-          dsp=False):
+def elspv(U, Y, na=0, nb=0, nc=0, pm0=0, maxiter=50, axcnv=None, xcnv=1e-8, afcnv=None, fcnv=1e-8, gcnv=None, dsp=False):
     N, r = Y.shape
-    if len(U) > 0:
-        N, m = U.shape
-    else:
-        m = 0
-    if len(Y) > 0:
-        N, r = Y.shape
-    else:
-        r = 0
-    if not N > 0:
-        print('In ELSPV: At least one data matrix should be not empty...')
-        return None
+    if len(U) > 0:  m = U.shape[1]
+    else:           m = 0
     if isinstance(na, (int, float)):  na = np.full([r, r], na)
     if isinstance(nb, (int, float)):  nb = np.full([r, m], nb)
     if isinstance(nc, (int, float)):  nc = np.full([r, r], nc)
     if isinstance(pm0, (int, float)): pm0 = np.full([r, 1], pm0)
     n = int(np.max(np.hstack((na, nb, nc))))
     nn = int(n - np.max(np.hstack((na, nb))))
-    arx = lspv(U, Y, na=na, nb=nb, pm0=pm0)
-    Ymab = lspv_apl(U[nn:], Y[nn:], arx, na=na, nb=nb, pm0=pm0)
-    if nc.size == 0:
-        E = np.empty((0, 0))
-        return arx, E
+
+    parx = lspv(U, Y, na=na, nb=nb, pm0=pm0)
+    Ymab = arx_apl(U[nn:], Y[nn:], parx, na=na, nb=nb, pm0=pm0)
+    # if nc.size == 0:  E = np.empty((0, 0));  return arx, E
     Fab = dmpv(U[nn:], Y[nn:], na=na, nb=nb, nc=np.zeros((nc.shape)), pm0=pm0)
     E = np.vstack((np.zeros([n, r]), Y[n:] - Ymab))
-    f = np.sum(np.diag(E.T@E) / N)
-    pm = np.vstack((arx, np.zeros((int(np.sum(nc)), 1))))
+    f = np.sum(np.diag(E.T@E)/N)
+    pm = np.vstack((parx, np.zeros((int(np.sum(nc)), 1))))
     Pm = np.vstack((pv2m(pm, na=na, nb=nb, pm0=pm0), np.zeros((r * int(np.max(nc)), r))))
     itr = 0
     iterate = 1
@@ -306,12 +319,6 @@ def elspv(U, Y, na=0, nb=0, nc=0, pm0=0, maxiter=50, axcnv=None, xcnv=1e-8, afcn
     pm = pm1 if f > f_1 else pm
     return pm, E
 ##############################################################################
-def elspv_apl(U, Y, E, pm, na=0, nb=0, nc=0, pm0=0):
-    F = dmpm(U, Y, E, na, nb, nc, pm0)
-    Pm = pv2m(pm, na, nb, nc, pm0)
-    Ym = F@Pm
-    return Ym
-##############################################################################
 # glspm():
 # glspm_apl():
 # glspv():
@@ -391,26 +398,49 @@ def lspm(U, Y, na=0, nb=0, pm0=False, w=None):
     else:
         F = dmpm(U, Y, na=na, nb=nb, nc=0, pm0=pm0)
         if w == None:
-            Pm = nsinv(F.T @ F) @ F.T @ Y[n:N][:]  # <<< think about Pseudoinverse
+            Pm = nsinv(F.T @ F) @ F.T @ Y[n:N][:]
         else:
             W = np.diag(w[n:N])
             Pm = nsinv(F.T @ W @ F) @ F.T @ W @ Y[n:N][:]
     return Pm
-
-
 ##############################################################################
-def lspm_apl(U, Y, Pm, na=0, nb=0, pm0=0):
-    if max(na, nb) == 0:
-        if pm0: U = np.hstack((np.ones((U.shape[0], 1)), U))
-        Ym = U@Pm
+def lin_apl(x, pm):
+    return x@pm
+##############################################################################
+def arx_apl(U, Y=None, pm=None, na=0, nb=0, pm0=0, sim=False, ltv=False, E=None):
+    if isinstance(nb, (int, float)):
+        r = pm.shape[1]
+        m = U.shape[1]
+        mtype = 'PM'
     else:
-        F = dmpm(U, Y, na=na, nb=nb, nc=0, pm0=pm0)
-        Ym = F@Pm
-    return Ym
-
-
+        r = nb.shape[0]
+        m = nb.shape[1]
+        mtype = 'PV'
+    if mtype == 'PV': pm = pv2m(pm, na, nb, pm0=pm0) # if model in PV form --> goto PM form
+    nn = np.max(np.hstack((na, nb)))
+    if not sim:
+        if nn == 0:
+            if pm0: U = np.hstack((np.ones((U.shape[0], 1)), U))
+            ym = lin_apl(U, pm)
+        else:
+            F = dmpm(U, Y, na=na, nb=nb, nc=0, pm0=pm0)
+            ym = lin_apl(F, pm)
+    else:
+        N = U.shape[0]
+        if not ltv: r = pm.shape[1]
+        else:       r = na.shape[0]
+        if E is None: E = zeros((N, r))
+        ym = np.zeros((N, r))
+        if ltv:
+            for k in range(nn, N):
+                fi = dmpm(U[k - nn:k + 1, :], ym[k - nn:k + 1, :], na=na, nb=nb, pm0=pm0)
+                ym[k, :] = fi@pv2m(pm[:, k], na, nb, pm0=pm0) + r_(E[k, :])
+        else:
+            for k in range(nn, N):
+                fi = dmpm(U[k - nn:k + 1, :], ym[k - nn:k + 1, :], na=na, nb=nb, pm0=pm0)
+                ym[k, :] = fi@pm + r_(E[k, :])
+    return ym
 ##############################################################################
-# def lspv(U,Y,par={'na':0, 'nb':0, 'nc':0, 'pm0':False}): # <<< Subject to change
 def lspv(U=np.empty((0, 0)),
          Y=np.empty((0, 0)),
          na=0,
@@ -439,12 +469,6 @@ def lspv(U=np.empty((0, 0)),
         pm = nsinv(F.T@F)@F.T@vec(Y[n:].T)
     return c_(pm)
 ##############################################################################
-def lspv_apl(U, Y, pm, na=None, nb=None, pm0=0):
-    Pm = pv2m(pm, na, nb, pm0=pm0)
-    F = dmpm(U, Y, na=na, nb=nb, pm0=pm0)
-    Ym = F@Pm
-    return Ym
-##############################################################################
 def pm2v(Pm, na=None, nb=None, nc=0, pm0=0):
     # todo: include nc and pm0
     r = m = 1
@@ -469,7 +493,6 @@ def pv2m(pm,
          nc=None,
          pm0=None,
          par={}):
-    # par={'na':0, 'nb':0, 'nc':0, 'pm0':False}):
     if not np.array([nb == None]).any():
         r, m = nb.shape
     elif not np.array([na == None]).any():
@@ -503,8 +526,9 @@ def pv2m(pm,
     nna = int(np.max(na)) if na.size > 0 else 0
     nnb = int(np.max(nb)) if nb.size > 0 else 0
     nnc = int(np.max(nc)) if nc.size > 0 else 0
-    nni = any(pm0)
+    nni = np.any(pm0)
     z = r * nna + m * nnb + r * nnc + nni
+    cnd = r == 1
     if z == 0:
         Pm = np.empty((0, 0))
         return Pm
@@ -517,7 +541,7 @@ def pv2m(pm,
                 ii += pm0[i]
         if nna:
             for j in range(0, r):
-                Pm[i, nni + j:int(na[i, j] * r - 1 + j):r] = pm[int(0 + ii):int(na[i, j] + ii), 0]
+                Pm[i, nni + j:int(na[i, j]*r - 1 + cnd + j):r] = pm[int(0 + ii):int(na[i, j] + ii), 0]
                 ii += na[i, j]
         if nnb:
             for j in range(0, m):
@@ -525,14 +549,14 @@ def pv2m(pm,
                 ii += nb[i, j]
         if nnc:
             for j in range(0, r):
-                Pm[i, nni + j + r * nna + m * nnb:int(nc[i, j] * r - 1 + j + r * nna + m * nnb):r] = pm[int(0 + ii):int(
+                Pm[i, nni + j + r * nna + m * nnb:int(nc[i, j] * r - 1 + cnd + j + r * nna + m * nnb):r] = pm[int(0 + ii):int(
                     nc[i, j] + ii), 0]
                 ii += nc[i, j]
     return Pm.T
 ##############################################################################
 # function Y = repmatc(X, m)
 # % Columnwise reproduction (m times) of X matrix, i.e. Y = [X X ... X]
-""" USE np.tile """
+# """ USE np.tile """
 # def repmatc(X, m):
 #     n = X.shape[1]
 #     n = size(X,2)
@@ -661,5 +685,5 @@ def roblspv(U, Y, na=0, nb=0, pm0=0, maxiter=50, dvaf=1e-2, hst=0):
     # ----------------------------------------------------------------
     return pm, st
 ##############################################################################
-# defv roblspv_apl():
+# def roblspv_apl():
 ##############################################################################
