@@ -1,5 +1,3 @@
-import copy
-
 import numpy as np
 import pandas as pd
 
@@ -7,118 +5,15 @@ from aislab.gnrl.sf import *
 from aislab.gnrl.measr import *
 
 ##############################################################################
-##############################################################################
-def func(X, args):
-    # Objective function for credit risk strategy optimization
-    # needed for old GA
-    case = 1
-    if isinstance(X, list):
-        case = -1
-        X = np.array(X)
-    # needed for old GA
-
-    from aislab.gnrl.sf import sort
-    from aislab.dp_feng.binenc import cut
-
-    data = args['data']  # data set
-    BRA = args['BRA']  # Desired Bad Rate in AA (less risky) risk zone
-    BRR = args['BRR']  # Desired Bad Rate in RR (most risky) risk zone
-    TA = args['TA']  # Desired Total number of applications in AA risk zone
-    TR = args['TR']  # Desired Total number of applications in RR risk zone
-    minNb = args['minNb']  # minimum number of Bad applications in each risk zone
-    minNgb = args['minNgb']  # minimum number of Good & Bad applications in each risk zone
-    mindBR = args['mindBR']  # minimum change in the Bad Rate moving from one risk zone to a neighbour zone
-    w = args['w']  # vector containing the weights of all business requirements included in the objective function
-    nc1 = args['nc1']  # number of cut-offs w.r.t. first (GB) scorecard
-    nc2 = args['nc2']  # number of cut-offs w.r.t. second (GBR) scorecard
-    lb1 = args['lb1']  # lower bound for cut-offs of the first scorecard
-    lb2 = args['lb2']  # lower bound for cut-offs of the second scorecard
-    ub1 = args['ub1']  # upper bound for cut-offs of the first scorecard
-    ub2 = args['ub2']  # upper bound for cut-offs of the second scorecard
-    minx = args['minx']  # upper bound for cut-offs of the first scorecard
-    maxx = args['maxx']  # upper bound for cut-offs of the second scorecard
-    if X.ndim == 1: X = np.array([X])
-    N = len(X)
-    F = np.full((N,), np.nan)
-    i = 0
-    for x in X:
-        x1, ss = sort(c_(x[:nc1, ]))
-        x2, ss = sort(c_(x[nc1:, ]))
-        minx = np.min([minx, np.min(x)])  # upper bound for cut-offs of the first scorecard
-        maxx = np.max([maxx, np.max(x)])  # upper bound for cut-offs of the second scorecard
-        x1 = (x1 - minx) / (maxx - minx) * (ub1 - lb1) + lb1
-        x2 = (x2 - minx) / (maxx - minx) * (ub2 - lb2) + lb2
-        x1 = np.round(x1.flatten())
-        x2 = np.round(x2.flatten())
-        if len(set(x1)) != len(x1) or len(set(x2)) != len(x2): return case*np.inf  # if there are 2 or more the same GB cut-offs: F = inf
-        try:
-            GB_zones = cut(data['Score_GB'].values, x1).flatten()
-            GBR_zones = cut(data['Score_GBR'].values, x2).flatten()
-        except:
-            print("In fobj(): Wrong cut-offs...")
-        good = confm(-GB_zones, -GBR_zones, w=data['Good'].values).astype(int)  # confusion matrix
-        bad = confm(-GB_zones, -GBR_zones, w=data['Bad'].values).astype(int)  # confusion matrix
-        reject = confm(-GB_zones, -GBR_zones, w=data['Reject'].values).astype(int)  # confusion matrix
-        import warnings
-        warnings.filterwarnings("ignore", category=RuntimeWarning)
-        bad_rate = bad / (good + bad) * 100
-        total_gb = good + bad
-        total = good + bad + reject
-        drBR = bad_rate[:, 1:] - bad_rate[:, :-1]  # rowwise delta BR
-        dcBR = bad_rate[1:, :] - bad_rate[:-1, :]  # colwise delta BR
-
-        mr, nr = np.shape(drBR)
-        mc, nc = np.shape(dcBR)
-        drBR1 = copy.deepcopy(drBR)
-        for k in range(nr-1, 0, -1):
-            for j in range(mr-1, 0, -1):
-                if k-1 >= 0 and not np.isnan(drBR1[j, k]) and np.isnan(drBR1[j, k-1]): drBR1[j, k-1] = drBR1[j, k]
-                if j-1 >= 0 and not np.isnan(drBR1[j, k]) and np.isnan(drBR1[j-1, k]): drBR1[j-1, k] = drBR1[j, k]
-        dcBR1 = copy.deepcopy(dcBR)
-        for k in range(nc-1, 0, -1):
-            for j in range(mc-1, 0, -1):
-                if k-1 >= 0 and not np.isnan(dcBR1[j, k]) and np.isnan(dcBR1[j, k-1]): dcBR1[j, k-1] = dcBR1[j, k]
-                if j-1 >= 0 and not np.isnan(dcBR1[j, k]) and np.isnan(dcBR1[j-1, k]): dcBR1[j-1, k] = dcBR1[j, k]
-        negdrBR = drBR1[drBR1 < mindBR]  # negative trend or not enough change in rowwise delta BR
-        negdcBR = dcBR1[dcBR1 < mindBR]  # negative trend or not enough change in colwise delta BR
-
-        # discriminatory power of the strategy  #  min(F1) = 0
-        F1 = (((BRA - bad_rate[0, 0]) / max(1e-6, 100 - BRA)) ** 2 + ((TA - total[0, 0]) / max(1e-6, TA)) ** 2) / 2
-        F2 = (((BRR - bad_rate[-1, -1]) / max(1e-6, BRR - 0)) ** 2 + ((TR - total[-1, -1]) / max(1e-6, TR)) ** 2) / 2
-        # monotonicity of BR  #  min(F2) = 0
-        F3 = ((np.sum(mindBR - negdrBR) + np.sum(mindBR - negdcBR)) / (nc1 * nc2)) ** 2
-        # mininmum number of Bad and Good & Bad applications per segment  #  min(F4) = min(F5) = 0
-        F4 = (np.sum((minNb - bad) * (minNb - bad > 0) * (total_gb > 0)) / max(1e-6,
-                                                                               (nc1 + 1) * (nc2 + 1) * minNb)) ** 2
-        F5 = (np.sum((minNgb - total_gb)*(minNgb - total_gb > 0)*(total_gb > 0))/max(1e-6, (nc1 + 1)*(nc2 + 1)*minNgb))**2
-        # minimum number of NaN cells: min(F6) = 0
-        F6 = (np.sum(np.sum(np.isnan(bad_rate))) / ((nc1 + 1) * (nc2 + 1))) ** 2
-        # minimum number of records in most populated cell  # min(F7) = 0
-        F7 = np.max(np.max(total)) / max(1e-6, sum(sum(total)))
-        F[i] = w[0] * F1 + w[1] * F2 + w[2] * F3 + w[3] * F4 + w[4] * F5 + w[5] * F6 + w[6] * F7
-
-        if np.isnan(F[i]):
-            F[i] = case * np.inf
-            return F
-        i += 1
-        # needed for old GA
-        if len(F) == 1: F = -F[0]
-        if isinstance(F, float) and np.isinf(F): F = -np.inf
-        # needed for old GA
-    return F
-##############################################################################
-##############################################################################
-
-##############################################################################
-def firstgen(pop, par, args):
-    F = [func(pop[x], args) for x in range(len(pop))]
+def firstgen(pop, fobj, args):
+    F = [fobj(pop[x], args) for x in range(len(pop))]
     sorted_F = sorted([[pop[x], F[x]] for x in range(len(pop))], key=lambda x: x[1])
     population = [sorted_F[x][0] for x in range(len(sorted_F))]
     F = [sorted_F[x][1] for x in range(len(sorted_F))]
     return {'individuals': population, 'F': sorted(F)}
 
 ##############################################################################
-def func_sim(max_F, itrConstF):
+def fobj_sim(max_F, itrConstF):
     result = False
     br = 0
     for i in range(len(max_F) - 1):
@@ -130,14 +25,14 @@ def func_sim(max_F, itrConstF):
     return result
 
 ##############################################################################
-def genopt(par, args):
-    itrConstF = par['stop_itrConstF']
+def genopt(fobj, args, Nind=100, mut_rate=50, mut_strength=2, mut_met='Gauss', mut_stdev=70, stop_itrConstF=10, pairing_met='Fbest', select_met='Fhalf'):
+    itrConstF = stop_itrConstF
     w = args['w']  # Acc  Rej  mt    Nb   Ngb  NaN  min_maxCell
     nc1 = args['nc1']
     w = w/np.sum(w)
-    pop = population(par, args, 0)
+    pop = population(Nind, args, 0)
     gen = []
-    gen.append(firstgen(pop, par, args))
+    gen.append(firstgen(pop, fobj, args))
     fitness_avg = np.array([sum(gen[0]['F'])/len(gen[0]['F'])])
     fitness_max = np.array([max(gen[0]['F'])])
     finish = False
@@ -148,8 +43,8 @@ def genopt(par, args):
         if itr == maxiter:                              break
         if max(fitness_max) >= 0:                       break
         if max(fitness_avg) >= 0:                       break
-        if func_sim(fitness_max, itrConstF) == True:    break
-        gen.append(nextgen(gen[-1], par, args, itr))
+        if fobj_sim(fitness_max, itrConstF) == True:    break
+        gen.append(nextgen(gen[-1], fobj, args, itr, Nind, select_met, pairing_met, mut_rate, mut_met, mut_strength, mut_stdev))
         fitness_avg = np.append(fitness_avg, sum(gen[-1]['F']) / len(gen[-1]['F']))
         fitness_max = np.append(fitness_max, max(gen[-1]['F']))
         print(itr, 'Fmin = ', -gen[-1]['F'][-1])
@@ -205,21 +100,18 @@ def mating(parents, seed, args, met='Single Point'):
     return child
 
 ##############################################################################
-def mutation(indv, par, args, seed, i_mut, i):
+def mutation(indv, args, seed, i_mut, i, mut_met, mut_strength, mut_stdev):
     if not any(i_mut == i): return indv
     nc1 = args['nc1']
     nc2 = args['nc2']
     minx = args['minx']
     maxx = args['maxx']
-    met = par['mut_met']
-    mut_strength = par['mut_strength']
-    stdev = par['mut_stdev']
     ind = rand(size=(mut_strength,), l=0, h=nc1+nc2-1, tp='int', seed=seed)
-    if met == 'Gauss':
+    if mut_met == 'Gauss':
         for x in ind:
             seed = seed*x + 1
-            indv[x] = int(indv[x] + randn(m=0, s=stdev, seed=seed))
-    if met == 'Reset':
+            indv[x] = int(indv[x] + randn(m=0, s=mut_stdev, seed=seed))
+    if mut_met == 'Reset':
         for x in ind:
             seed = seed*x + 1
             indv[x] = rand(l=minx, h = maxx, tp='int', seed=seed)[0, 0]
@@ -227,16 +119,12 @@ def mutation(indv, par, args, seed, i_mut, i):
 
 
 ##############################################################################
-def nextgen(gen, par, args, itr):
-    Nind = par['Nind']
-    selection_met = par['select_met']
-    pairing_met = par['pairing_met']
-    mut_rate = par['mut_rate']
+def nextgen(gen, fobj, args, itr, Nind, select_met, pairing_met, mut_rate, mut_met, mut_strength, mut_stdev):
     best = {}
     next_sgen = {}
     best['individuals'] = gen['individuals'].pop(-1)
     best['F'] = gen['F'].pop(-1)
-    selected = selection(gen, itr, selection_met)
+    selected = selection(gen, itr, select_met)
     parents = pairing(best, selected, itr, met=pairing_met)
     children = [[[mating(parents[i], itr + i + j, args) for i in range(len(parents))]
                  [j][h] for h in range(2)]
@@ -246,9 +134,9 @@ def nextgen(gen, par, args, itr):
     unmutated = selected['individuals'] + children1 + children2
     n = len(gen['individuals'])
     i_mut = find(rand(size=(n,), seed=itr) < mut_rate / 100)
-    mutated = [mutation(unmutated[i], par, args, itr * Nind + i, i_mut, i) for i in range(n)]
+    mutated = [mutation(unmutated[i], args, itr * Nind + i, i_mut, i, mut_met, mut_strength, mut_stdev) for i in range(n)]
     indvs = mutated + [best['individuals']]
-    next_gen = [func(mutated[i], args) for i in range(len(mutated))]
+    next_gen = [fobj(mutated[i], args) for i in range(len(mutated))]
     F = [next_gen[x] for x in range(len(gen['F']))] + [best['F']]
     sorted_next_gen = sorted([[indvs[x], F[x]] for x in range(len(indvs))], key=lambda x: x[1])
     next_sgen['individuals'] = [sorted_next_gen[x][0] for x in range(len(sorted_next_gen))]
@@ -291,8 +179,7 @@ def pairing(best, selected, seed, met='Fbest'):
 
 
 ##############################################################################
-def population(par, args, itr):
-    Nind = par['Nind']
+def population(Nind, args, itr):
     pop = []
     for i in range(Nind):
         args['seed1'] = (itr + i) * (args['nc1'] + args['nc2'])
